@@ -27,11 +27,8 @@
 #include <sys/types.h>
 #include <gst/gst.h>
 
-#include "config.h"
-
 G_BEGIN_DECLS
 
-GstElementDetails gst_pipefilter_details;
 
 #define GST_TYPE_PIPEFILTER \
   (gst_pipefilter_get_type())
@@ -47,7 +44,7 @@ GstElementDetails gst_pipefilter_details;
 typedef enum {
   GST_PIPEFILTER_OPEN		= GST_ELEMENT_FLAG_LAST,
 
-  GST_PIPEFILTER_FLAG_LAST	= GST_ELEMENT_FLAG_LAST,
+  GST_PIPEFILTER_FLAG_LAST	= GST_ELEMENT_FLAG_LAST + 2
 } GstPipeFilterFlags;
 
 typedef struct _GstPipefilter GstPipefilter;
