@@ -32,6 +32,16 @@
 
 #include "gstscheduler.h"
 #include "gstindex.h"
+#include "gstutils.h"
+
+GST_DEBUG_CATEGORY_STATIC (bin_debug);
+#define GST_CAT_DEFAULT bin_debug
+#define GST_LOG_BIN_CONTENTS(bin, text) GST_LOG_OBJECT ((bin), \
+	text ": %d elements: %u PLAYING, %u PAUSED, %u READY, %u NULL, own state: %s", \
+	(bin)->numchildren, (guint) (bin)->child_states[3], \
+	(guint) (bin)->child_states[2], (bin)->child_states[1], \
+	(bin)->child_states[0], gst_element_state_get_name (GST_STATE (bin)))
+
 
 static GstElementDetails gst_bin_details = GST_ELEMENT_DETAILS ("Generic bin",
     "Generic/Bin",
@@ -56,6 +66,8 @@ static void gst_bin_add_func (GstBin * bin, GstElement * element);
 static void gst_bin_remove_func (GstBin * bin, GstElement * element);
 static void gst_bin_child_state_change_func (GstBin * bin,
     GstElementState oldstate, GstElementState newstate, GstElement * child);
+GstElementStateReturn gst_bin_set_state (GstElement * element,
+    GstElementState state);
 
 static GstClock *gst_bin_get_clock_func (GstElement * element);
 static void gst_bin_set_clock_func (GstElement * element, GstClock * clock);
@@ -89,6 +101,11 @@ static void gst_bin_init (GstBin * bin);
 static GstElementClass *parent_class = NULL;
 static guint gst_bin_signals[LAST_SIGNAL] = { 0 };
 
+/**
+ * gst_bin_get_type:
+ *
+ * Returns: the type of #GstBin
+ */
 GType
 gst_bin_get_type (void)
 {
@@ -101,13 +118,16 @@ gst_bin_get_type (void)
       NULL,
       NULL,
       sizeof (GstBin),
-      8,
+      0,
       (GInstanceInitFunc) gst_bin_init,
       NULL
     };
 
     _gst_bin_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstBin", &bin_info, 0);
+
+    GST_DEBUG_CATEGORY_INIT (bin_debug, "bin", GST_DEBUG_BOLD,
+        "debugging info for the 'bin' container element");
   }
   return _gst_bin_type;
 }
@@ -156,6 +176,7 @@ gst_bin_class_init (GstBinClass * klass)
 #endif
 
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_bin_change_state);
+  gstelement_class->set_state = GST_DEBUG_FUNCPTR (gst_bin_set_state);
 #ifndef GST_DISABLE_INDEX
   gstelement_class->set_index = GST_DEBUG_FUNCPTR (gst_bin_set_index);
 #endif
@@ -305,42 +326,42 @@ gst_bin_set_element_sched (GstElement * element, GstScheduler * sched)
     /* set the children's schedule */
     g_list_foreach (GST_BIN (element)->children,
         (GFunc) gst_bin_set_element_sched, sched);
-  }
-  /* otherwise, if it's just a regular old element */
-  else {
+  } else {
+    /* otherwise, if it's just a regular old element */
     GList *pads;
 
     gst_scheduler_add_element (sched, element);
 
-    /* set the sched pointer in all the pads */
-    pads = element->pads;
-    while (pads) {
-      GstPad *pad;
+    if (!GST_FLAG_IS_SET (element, GST_ELEMENT_DECOUPLED)) {
+      /* set the sched pointer in all the pads */
+      pads = element->pads;
+      while (pads) {
+        GstPad *pad;
 
-      pad = GST_PAD (pads->data);
-      pads = g_list_next (pads);
+        pad = GST_PAD (pads->data);
+        pads = g_list_next (pads);
 
-      /* we only operate on real pads */
-      if (!GST_IS_REAL_PAD (pad))
-        continue;
+        /* we only operate on real pads */
+        if (!GST_IS_REAL_PAD (pad))
+          continue;
 
-      /* if the peer element exists and is a candidate */
-      if (GST_PAD_PEER (pad)) {
-        if (gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
-          GST_CAT_LOG (GST_CAT_SCHEDULING,
-              "peer is in same scheduler, telling scheduler");
+        /* if the peer element exists and is a candidate */
+        if (GST_PAD_PEER (pad)) {
+          if (gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
+            GST_CAT_LOG (GST_CAT_SCHEDULING,
+                "peer is in same scheduler, telling scheduler");
 
-          if (GST_PAD_IS_SRC (pad))
-            gst_scheduler_pad_link (sched, pad, GST_PAD_PEER (pad));
-          else
-            gst_scheduler_pad_link (sched, GST_PAD_PEER (pad), pad);
+            if (GST_PAD_IS_SRC (pad))
+              gst_scheduler_pad_link (sched, pad, GST_PAD_PEER (pad));
+            else
+              gst_scheduler_pad_link (sched, GST_PAD_PEER (pad), pad);
+          }
         }
       }
     }
   }
 }
 
-
 static void
 gst_bin_unset_element_sched (GstElement * element, GstScheduler * sched)
 {
@@ -370,36 +391,38 @@ gst_bin_unset_element_sched (GstElement * element, GstScheduler * sched)
         (GFunc) gst_bin_unset_element_sched, sched);
 
     gst_scheduler_remove_element (GST_ELEMENT_SCHED (element), element);
-  }
-  /* otherwise, if it's just a regular old element */
-  else {
+  } else {
+    /* otherwise, if it's just a regular old element */
     GList *pads;
 
-    /* set the sched pointer in all the pads */
-    pads = element->pads;
-    while (pads) {
-      GstPad *pad;
+    if (!GST_FLAG_IS_SET (element, GST_ELEMENT_DECOUPLED)) {
+      /* unset the sched pointer in all the pads */
+      pads = element->pads;
+      while (pads) {
+        GstPad *pad;
 
-      pad = GST_PAD (pads->data);
-      pads = g_list_next (pads);
+        pad = GST_PAD (pads->data);
+        pads = g_list_next (pads);
 
-      /* we only operate on real pads */
-      if (!GST_IS_REAL_PAD (pad))
-        continue;
+        /* we only operate on real pads */
+        if (!GST_IS_REAL_PAD (pad))
+          continue;
 
-      /* if the peer element exists and is a candidate */
-      if (GST_PAD_PEER (pad)) {
-        if (gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
-          GST_CAT_LOG (GST_CAT_SCHEDULING,
-              "peer is in same scheduler, telling scheduler");
+        /* if the peer element exists and is a candidate */
+        if (GST_PAD_PEER (pad)) {
+          if (gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
+            GST_CAT_LOG (GST_CAT_SCHEDULING,
+                "peer is in same scheduler, telling scheduler");
 
-          if (GST_PAD_IS_SRC (pad))
-            gst_scheduler_pad_unlink (sched, pad, GST_PAD_PEER (pad));
-          else
-            gst_scheduler_pad_unlink (sched, GST_PAD_PEER (pad), pad);
+            if (GST_PAD_IS_SRC (pad))
+              gst_scheduler_pad_unlink (sched, pad, GST_PAD_PEER (pad));
+            else
+              gst_scheduler_pad_unlink (sched, GST_PAD_PEER (pad), pad);
+          }
         }
       }
     }
+
     gst_scheduler_remove_element (GST_ELEMENT_SCHED (element), element);
   }
 }
@@ -409,9 +432,10 @@ gst_bin_unset_element_sched (GstElement * element, GstScheduler * sched)
  * gst_bin_add_many:
  * @bin: the bin to add the elements to
  * @element_1: the first element to add to the bin
- * @...: NULL-terminated list of elements to add to the bin
+ * @...: additional elements to add to the bin
  * 
- * Add a list of elements to a bin. Uses #gst_bin_add.
+ * Adds a NULL-terminated list of elements to a bin.  This function is
+ * equivalent to calling #gst_bin_add() for each member of the list.
  */
 void
 gst_bin_add_many (GstBin * bin, GstElement * element_1, ...)
@@ -450,6 +474,12 @@ gst_bin_add_func (GstBin * bin, GstElement * element)
     return;
   }
 
+  if (GST_STATE (element) > GST_STATE (bin)) {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, bin,
+        "setting state to receive element \"%s\"", GST_OBJECT_NAME (element));
+    gst_element_set_state ((GstElement *) bin, GST_STATE (element));
+  }
+
   /* set the element's parent and add the element to the bin's list of children */
   gst_object_set_parent (GST_OBJECT (element), GST_OBJECT (bin));
 
@@ -458,7 +488,7 @@ gst_bin_add_func (GstBin * bin, GstElement * element)
 
   /* bump our internal state counter */
   state = GST_STATE (element);
-  while (state >>= 1)
+  while ((state >>= 1) != 0)
     state_idx++;
   bin->child_states[state_idx]++;
 
@@ -481,8 +511,8 @@ gst_bin_add_func (GstBin * bin, GstElement * element)
  * @bin: #GstBin to add element to
  * @element: #GstElement to add to bin
  *
- * Add the given element to the bin.  Set the elements parent, and thus
- * add a reference.
+ * Adds the given element to the bin.  Sets the element's parent, and thus
+ * takes ownership of the element. An element can only be added to one bin.
  */
 void
 gst_bin_add (GstBin * bin, GstElement * element)
@@ -531,7 +561,7 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
 
   /* bump our internal state counter */
   state = GST_STATE (element);
-  while (state >>= 1)
+  while ((state >>= 1) != 0)
     state_idx++;
   bin->child_states[state_idx]--;
 
@@ -543,9 +573,17 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
   gst_object_unparent (GST_OBJECT (element));
 
   /* if we're down to zero children, force state to NULL */
-  if (bin->numchildren == 0 && GST_ELEMENT_SCHED (bin) != NULL) {
-    GST_STATE_PENDING (bin) = GST_STATE_NULL;
+  while (bin->numchildren == 0 && GST_ELEMENT_SCHED (bin) != NULL &&
+      GST_STATE (bin) > GST_STATE_NULL) {
+    GstElementState next = GST_STATE (bin) >> 1;
+
+    GST_STATE_PENDING (bin) = next;
     gst_bin_change_state_norecurse (bin);
+    if (!GST_STATE (bin) == next) {
+      g_warning ("bin %s failed state change to %d", GST_ELEMENT_NAME (bin),
+          next);
+      break;
+    }
   }
   g_signal_emit (G_OBJECT (bin), gst_bin_signals[ELEMENT_REMOVED], 0, element);
 
@@ -558,10 +596,11 @@ gst_bin_remove_func (GstBin * bin, GstElement * element)
  * @bin: #GstBin to remove element from
  * @element: #GstElement to remove
  *
- * Remove the element from its associated bin, unparenting as well.
- * The element will also be unreferenced so there's no need to call
- * gst_object_unref on it.
- * If you want the element to still exist after removing, you need to call
+ * Remove the element from its associated bin, unparenting it as well.
+ * Unparenting the element means that the element will be dereferenced,
+ * so if the bin holds the only reference to the element, the element
+ * will be freed in the process of removing it from the bin.  If you
+ * want the element to still exist after removing, you need to call
  * #gst_object_ref before removing it from the bin.
  */
 void
@@ -574,7 +613,6 @@ gst_bin_remove (GstBin * bin, GstElement * element)
 
   g_return_if_fail (GST_IS_BIN (bin));
   g_return_if_fail (GST_IS_ELEMENT (element));
-  g_return_if_fail (bin->children != NULL);
 
   bclass = GST_BIN_GET_CLASS (bin);
 
@@ -591,7 +629,8 @@ gst_bin_remove (GstBin * bin, GstElement * element)
  * @element_1: the first element to remove from the bin
  * @...: NULL-terminated list of elements to remove from the bin
  * 
- * Remove a list of elements from a bin. Uses #gst_bin_remove.
+ * Remove a list of elements from a bin. This function is equivalent
+ * to calling #gst_bin_remove with each member of the list.
  */
 void
 gst_bin_remove_many (GstBin * bin, GstElement * element_1, ...)
@@ -650,14 +689,18 @@ static void
 gst_bin_child_state_change_func (GstBin * bin, GstElementState oldstate,
     GstElementState newstate, GstElement * child)
 {
+  GstElementState old = 0, new = 0;
   gint old_idx = 0, new_idx = 0, i;
 
-  while (oldstate >>= 1)
+  old = oldstate;
+  new = newstate;
+  while ((old >>= 1) != 0)
     old_idx++;
-  while (newstate >>= 1)
+  while ((new >>= 1) != 0)
     new_idx++;
 
   GST_LOCK (bin);
+  GST_LOG_BIN_CONTENTS (bin, "before child state change");
   bin->child_states[old_idx]--;
   bin->child_states[new_idx]++;
 
@@ -676,24 +719,120 @@ gst_bin_child_state_change_func (GstBin * bin, GstElementState oldstate,
           g_warning ("%s: state change in callback %d %d",
               GST_ELEMENT_NAME (bin), state, GST_STATE (bin));
         }
+        GST_LOG_BIN_CONTENTS (bin, "after child state change");
         return;
       }
       break;
     }
   }
+  GST_LOG_BIN_CONTENTS (bin, "after child state change");
   GST_UNLOCK (bin);
 }
 
+typedef gboolean (*GstBinForeachFunc) (GstBin * bin, GstElement * element,
+    gpointer data);
+
+/*
+ * gst_bin_foreach:
+ * @bin: bin to traverse
+ * @func: function to call on each child
+ * @data: user data handed to each function call
+ *
+ * Calls @func on each child of the bin. If @func returns FALSE, 
+ * gst_bin_foreach() immediately returns.
+ * It is assumed that calling @func may alter the set of @bin's children. @func
+ * will only be called on children that were in @bin when gst_bin_foreach() was
+ * called, and that are still in @bin when the child is reached.
+ *
+ * Returns: TRUE if @func always returned TRUE, FALSE otherwise
+ **/
+static gboolean
+gst_bin_foreach (GstBin * bin, GstBinForeachFunc func, gpointer data)
+{
+  GList *kids, *walk;
+
+  g_return_val_if_fail (GST_IS_BIN (bin), FALSE);
+  g_return_val_if_fail (func != NULL, FALSE);
+
+  kids = g_list_copy (bin->children);
+
+  for (walk = kids; walk; walk = g_list_next (walk)) {
+    GstElement *element = (GstElement *) walk->data;
+
+    if (g_list_find (bin->children, element)) {
+      gboolean res = func (bin, element, data);
+
+      if (!res) {
+        g_list_free (kids);
+        return FALSE;
+      }
+    }
+  }
+
+  g_list_free (kids);
+  return TRUE;
+}
+
+typedef struct
+{
+  GstElementState pending;
+  GstElementStateReturn result;
+}
+SetKidStateData;
+static int
+set_kid_state_func (GstBin * bin, GstElement * child, gpointer user_data)
+{
+  GstElementState old_child_state;
+  SetKidStateData *data = user_data;
+
+  if (GST_FLAG_IS_SET (child, GST_ELEMENT_LOCKED_STATE)) {
+    return TRUE;
+  }
+
+  old_child_state = GST_STATE (child);
+
+  GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, bin,
+      "changing state of child %s from current %s to pending %s",
+      GST_ELEMENT_NAME (child), gst_element_state_get_name (old_child_state),
+      gst_element_state_get_name (data->pending));
+
+  switch (gst_element_set_state (child, data->pending)) {
+    case GST_STATE_FAILURE:
+      GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin,
+          "child '%s' failed to go to state %d(%s)",
+          GST_ELEMENT_NAME (child),
+          data->pending, gst_element_state_get_name (data->pending));
+
+      gst_element_set_state (child, old_child_state);
+      return FALSE;             /* error out to the caller */
+
+    case GST_STATE_ASYNC:
+      GST_CAT_INFO_OBJECT (GST_CAT_STATES, bin,
+          "child '%s' is changing state asynchronously",
+          GST_ELEMENT_NAME (child));
+      data->result = GST_STATE_ASYNC;
+      return TRUE;
+
+    case GST_STATE_SUCCESS:
+      GST_CAT_DEBUG (GST_CAT_STATES,
+          "child '%s' changed state to %d(%s) successfully",
+          GST_ELEMENT_NAME (child), data->pending,
+          gst_element_state_get_name (data->pending));
+      return TRUE;
+
+    default:
+      g_assert_not_reached ();
+      return FALSE;             /* satisfy gcc */
+  }
+}
+
 static GstElementStateReturn
 gst_bin_change_state (GstElement * element)
 {
   GstBin *bin;
-  GList *children;
-  GstElement *child;
   GstElementStateReturn ret;
   GstElementState old_state, pending;
-  gint transition;
-  gboolean have_async = FALSE;
+  SetKidStateData data;
 
   g_return_val_if_fail (GST_IS_BIN (element), GST_STATE_FAILURE);
 
@@ -701,7 +840,6 @@ gst_bin_change_state (GstElement * element)
 
   old_state = GST_STATE (element);
   pending = GST_STATE_PENDING (element);
-  transition = GST_STATE_TRANSITION (element);
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
       "changing state of children from %s to %s",
@@ -711,60 +849,11 @@ gst_bin_change_state (GstElement * element)
   if (pending == GST_STATE_VOID_PENDING)
     return GST_STATE_SUCCESS;
 
-  if (old_state == pending) {
-    GST_CAT_LOG_OBJECT (GST_CAT_STATES, element,
-        "old and pending state are both %s, returning",
-        gst_element_state_get_name (pending));
-    return GST_STATE_SUCCESS;
-  }
-
-  children = bin->children;
-
-  while (children) {
-    GstElementState old_child_state;
-
-    child = GST_ELEMENT (children->data);
-    children = g_list_next (children);
-
-    if (GST_FLAG_IS_SET (child, GST_ELEMENT_LOCKED_STATE))
-      continue;
-
-    old_child_state = GST_STATE (child);
-
-    GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
-        "changing state of child %s from current %s to pending %s",
-        GST_ELEMENT_NAME (child), gst_element_state_get_name (old_child_state),
-        gst_element_state_get_name (pending));
-
-    switch (gst_element_set_state (child, pending)) {
-      case GST_STATE_FAILURE:
-        GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
-            "child '%s' failed to go to state %d(%s)",
-            GST_ELEMENT_NAME (child),
-            pending, gst_element_state_get_name (pending));
-
-        gst_element_set_state (child, old_child_state);
-        /* There was a check for elements being in the same scheduling group
-           here. Removed by dolphy <julien@moutte.net>. No matter the 
-           scheduling group we should always return a failure. This change
-           seems to work on my machine and fixes tons of issues. If anyone
-           want to revert please tell me what it breaks first, Thanks. */
-        GST_STATE_PENDING (element) = old_state;
-        return GST_STATE_FAILURE;
-        break;
-      case GST_STATE_ASYNC:
-        GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
-            "child '%s' is changing state asynchronously",
-            GST_ELEMENT_NAME (child));
-        have_async = TRUE;
-        break;
-      case GST_STATE_SUCCESS:
-        GST_CAT_DEBUG (GST_CAT_STATES,
-            "child '%s' changed state to %d(%s) successfully",
-            GST_ELEMENT_NAME (child), pending,
-            gst_element_state_get_name (pending));
-        break;
-    }
+  data.pending = pending;
+  data.result = GST_STATE_SUCCESS;
+  if (!gst_bin_foreach (bin, set_kid_state_func, &data)) {
+    GST_STATE_PENDING (element) = old_state;
+    return GST_STATE_FAILURE;
   }
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
@@ -773,9 +862,10 @@ gst_bin_change_state (GstElement * element)
       gst_element_state_get_name (pending),
       gst_element_state_get_name (GST_STATE (element)));
 
-  if (have_async)
+  if (data.result == GST_STATE_ASYNC)
     ret = GST_STATE_ASYNC;
   else {
+    /* FIXME: this should have been done by the children already, no? */
     if (parent_class->change_state) {
       ret = parent_class->change_state (element);
     } else
@@ -784,6 +874,26 @@ gst_bin_change_state (GstElement * element)
   return ret;
 }
 
+GstElementStateReturn
+gst_bin_set_state (GstElement * element, GstElementState state)
+{
+  GstBin *bin = GST_BIN (element);
+
+  if (GST_STATE (bin) == state) {
+    SetKidStateData data;
+
+    data.pending = state;
+    data.result = GST_STATE_SUCCESS;
+    if (!gst_bin_foreach (bin, set_kid_state_func, &data)) {
+      return GST_STATE_FAILURE;
+    } else {
+      return data.result;
+    }
+  } else {
+    return GST_CALL_PARENT_WITH_DEFAULT (GST_ELEMENT_CLASS, set_state, (element,
+            state), GST_STATE_FAILURE);
+  }
+}
 
 static GstElementStateReturn
 gst_bin_change_state_norecurse (GstBin * bin)
@@ -803,26 +913,16 @@ static void
 gst_bin_dispose (GObject * object)
 {
   GstBin *bin = GST_BIN (object);
-  GList *children, *orig;
-  GstElement *child;
 
-  GST_CAT_DEBUG (GST_CAT_REFCOUNTING, "dispose");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
 
-  if (gst_element_get_state (GST_ELEMENT (object)) == GST_STATE_PLAYING)
-    gst_element_set_state (GST_ELEMENT (object), GST_STATE_PAUSED);
+  gst_element_set_state (GST_ELEMENT (object), GST_STATE_NULL);
 
-  if (bin->children) {
-    orig = children = g_list_copy (bin->children);
-    while (children) {
-      child = GST_ELEMENT (children->data);
-      gst_bin_remove (bin, child);
-      children = g_list_next (children);
-    }
-    g_list_free (bin->children);
-    g_list_free (orig);
+  while (bin->children) {
+    gst_bin_remove (bin, GST_ELEMENT (bin->children->data));
   }
-  bin->children = NULL;
-  bin->numchildren = 0;
+  g_assert (bin->children == NULL);
+  g_assert (bin->numchildren == 0);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -920,10 +1020,10 @@ gst_bin_get_list (GstBin * bin)
  * @bin: bin to find element in
  * @interface: interface to be implemented by interface
  *
- * Looks for the first element inside the bin that implements the given 
+ * Looks for the first element inside the bin that implements the given
  * interface. If such an element is found, it returns the element. You can
  * cast this element to the given interface afterwards.
- * If you want all elements that implement the interface, use 
+ * If you want all elements that implement the interface, use
  * gst_bin_get_all_by_interface(). The function recurses bins inside bins.
  *
  * Returns: An element inside the bin implementing the interface.
@@ -958,9 +1058,9 @@ gst_bin_get_by_interface (GstBin * bin, GType interface)
  * @bin: bin to find elements in
  * @interface: interface to be implemented by interface
  *
- * Looks for all element inside the bin that implements the given 
+ * Looks for all elements inside the bin that implements the given
  * interface. You can safely cast all returned elements to the given interface.
- * The function recurses bins inside bins. You need to free the list using 
+ * The function recurses bins inside bins. You need to free the list using
  * g_list_free() after use.
  *
  * Returns: An element inside the bin implementing the interface.
@@ -975,8 +1075,11 @@ gst_bin_get_all_by_interface (GstBin * bin, GType interface)
 
   walk = bin->children;
   while (walk) {
-    if (G_TYPE_CHECK_INSTANCE_TYPE (walk->data, interface))
+    if (G_TYPE_CHECK_INSTANCE_TYPE (walk->data, interface)) {
+      GST_DEBUG_OBJECT (bin, "element %s implements requested interface",
+          GST_ELEMENT_NAME (GST_ELEMENT (walk->data)));
       ret = g_list_prepend (ret, walk->data);
+    }
     if (GST_IS_BIN (walk->data)) {
       ret = g_list_concat (ret,
           gst_bin_get_all_by_interface (GST_BIN (walk->data), interface));
@@ -1028,6 +1131,7 @@ gst_bin_sync_children_state (GstBin * bin)
           break;
         case GST_STATE_FAILURE:
           ret = GST_STATE_FAILURE;
+          break;
         default:
           /* make sure gst_element_set_state never returns this */
           g_assert_not_reached ();
@@ -1095,30 +1199,60 @@ gst_bin_restore_thyself (GstObject * object, xmlNodePtr self)
 
     field = field->next;
   }
+  if (GST_OBJECT_CLASS (parent_class)->restore_thyself)
+    (GST_OBJECT_CLASS (parent_class)->restore_thyself) (object, self);
 }
 #endif /* GST_DISABLE_LOADSAVE */
 
+static GStaticRecMutex iterate_lock = G_STATIC_REC_MUTEX_INIT;
+
 static gboolean
 gst_bin_iterate_func (GstBin * bin)
 {
+  GstScheduler *sched = GST_ELEMENT_SCHED (bin);
+
+  g_static_rec_mutex_unlock (&iterate_lock);
+
   /* only iterate if this is the manager bin */
-  if (GST_ELEMENT_SCHED (bin) &&
-      GST_ELEMENT_SCHED (bin)->parent == GST_ELEMENT (bin)) {
+  if (sched && sched->parent == GST_ELEMENT (bin)) {
     GstSchedulerState state;
 
-    state = gst_scheduler_iterate (GST_ELEMENT_SCHED (bin));
+    state = gst_scheduler_iterate (sched);
 
     if (state == GST_SCHEDULER_STATE_RUNNING) {
-      return TRUE;
+      goto done;
     } else if (state == GST_SCHEDULER_STATE_ERROR) {
       gst_element_set_state (GST_ELEMENT (bin), GST_STATE_PAUSED);
+    } else if (state == GST_SCHEDULER_STATE_STOPPED) {
+      /* check if we have children scheds that are still running */
+      /* FIXME: remove in 0.9? autouseless because iterations gone? */
+      GList *walk;
+
+      for (walk = sched->schedulers; walk; walk = g_list_next (walk)) {
+        GstScheduler *test = walk->data;
+
+        g_return_val_if_fail (test->parent, FALSE);
+        if (GST_STATE (test->parent) == GST_STATE_PLAYING) {
+          GST_CAT_DEBUG_OBJECT (GST_CAT_SCHEDULING, bin,
+              "current bin is not iterating, but children are, "
+              "so returning TRUE anyway...");
+          g_usleep (1);
+          goto done;
+        }
+      }
     }
   } else {
     g_warning ("bin \"%s\" is not the managing bin, can't be iterated on!\n",
         GST_ELEMENT_NAME (bin));
   }
 
+  g_static_rec_mutex_lock (&iterate_lock);
+
   return FALSE;
+
+done:
+  g_static_rec_mutex_lock (&iterate_lock);
+  return TRUE;
 }
 
 /**
@@ -1138,26 +1272,16 @@ gst_bin_iterate (GstBin * bin)
   g_return_val_if_fail (bin != NULL, FALSE);
   g_return_val_if_fail (GST_IS_BIN (bin), FALSE);
 
-  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, bin, "starting iteration");
-
+  GST_CAT_LOG_OBJECT (GST_CAT_SCHEDULING, bin, "starting iteration");
   gst_object_ref (GST_OBJECT (bin));
 
+  g_static_rec_mutex_lock (&iterate_lock);
   running = FALSE;
   g_signal_emit (G_OBJECT (bin), gst_bin_signals[ITERATE], 0, &running);
+  g_static_rec_mutex_unlock (&iterate_lock);
 
-  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, bin, "finished iteration");
-
-  if (!running) {
-    if (GST_STATE (bin) == GST_STATE_PLAYING &&
-        GST_STATE_PENDING (bin) == GST_STATE_VOID_PENDING) {
-      GST_CAT_DEBUG (GST_CAT_DATAFLOW,
-          "[%s]: polling for child shutdown after useless iteration",
-          GST_ELEMENT_NAME (bin));
-      g_usleep (1);
-      running = TRUE;
-    }
-  }
   gst_object_unref (GST_OBJECT (bin));
+  GST_CAT_LOG_OBJECT (GST_CAT_SCHEDULING, bin, "finished iteration");
 
   return running;
 }
