@@ -22,7 +22,6 @@
  */
 
 #include "gst_private.h"
-
 #include "gstinfo.h"
 
 #ifndef GST_DISABLE_GST_DEBUG
@@ -34,11 +33,10 @@
 #include <printf.h>
 #endif
 #include <stdio.h>              /* fprintf */
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <string.h>             /* G_VA_COPY */
-#include "gstinfo.h"
-#include "gstlog.h"
-#include "gst_private.h"
 #include "gstelement.h"
 #include "gstpad.h"
 #include "gstscheduler.h"
@@ -47,7 +45,8 @@
 #include <valgrind/valgrind.h>
 #endif
 
-GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEBUG);
+/* underscore is to prevent conflict with GST_CAT_DEBUG define */
+GST_DEBUG_CATEGORY_STATIC (_GST_CAT_DEBUG);
 
 #if 0
 #if defined __sgi__
@@ -78,8 +77,6 @@ dladdr (void *address, Dl_info * dl)
 #endif /* __sgi__ */
 #endif
 
-extern gchar *_gst_progname;
-
 static void gst_debug_reset_threshold (gpointer category, gpointer unused);
 static void gst_debug_reset_all_thresholds (void);
 
@@ -137,6 +134,8 @@ GstDebugCategory *GST_CAT_PARENTAGE = NULL;
 GstDebugCategory *GST_CAT_STATES = NULL;
 GstDebugCategory *GST_CAT_PLANNING = NULL;
 GstDebugCategory *GST_CAT_SCHEDULING = NULL;
+
+/* FIXME: remove GST_CAT_DATAFLOW in 0.9 */
 GstDebugCategory *GST_CAT_DATAFLOW = NULL;
 GstDebugCategory *GST_CAT_BUFFER = NULL;
 GstDebugCategory *GST_CAT_CAPS = NULL;
@@ -213,7 +212,7 @@ _gst_debug_init (void)
   /* do NOT use a single debug function before this line has been run */
   GST_CAT_DEFAULT = _gst_debug_category_new ("default",
       GST_DEBUG_UNDERLINE, NULL);
-  GST_CAT_DEBUG = _gst_debug_category_new ("GST_DEBUG",
+  _GST_CAT_DEBUG = _gst_debug_category_new ("GST_DEBUG",
       GST_DEBUG_BOLD | GST_DEBUG_FG_YELLOW, "debugging subsystem");
 
   gst_debug_add_log_function (gst_debug_log_default, NULL);
@@ -237,6 +236,7 @@ _gst_debug_init (void)
       GST_DEBUG_BOLD | GST_DEBUG_FG_MAGENTA, NULL);
   GST_CAT_SCHEDULING = _gst_debug_category_new ("GST_SCHEDULING",
       GST_DEBUG_BOLD | GST_DEBUG_FG_MAGENTA, NULL);
+/* FIXME: remove GST_CAT_DATAFLOW in 0.9 */
   GST_CAT_DATAFLOW = _gst_debug_category_new ("GST_DATAFLOW",
       GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, NULL);
   GST_CAT_BUFFER = _gst_debug_category_new ("GST_BUFFER",
@@ -284,7 +284,7 @@ _gst_debug_init (void)
 }
 
 /* we can't do this further above, because we initialize the GST_CAT_DEFAULT struct */
-#define GST_CAT_DEFAULT GST_CAT_DEBUG
+#define GST_CAT_DEFAULT _GST_CAT_DEBUG
 
 /**
  * gst_debug_log:
@@ -485,6 +485,7 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
   gchar *obj;
   gchar *pidcolor;
   gint pid;
+  GTimeVal now;
 
   if (level > gst_debug_category_get_threshold (category))
     return;
@@ -506,10 +507,12 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
 
   obj = object ? gst_debug_print_object (object) : g_strdup ("");
 
-  g_printerr ("%s %s%15s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
-      gst_debug_level_get_name (level),
-      color, gst_debug_category_get_name (category), clear,
-      pidcolor, pid, clear,
+  g_get_current_time (&now);
+  g_printerr ("%s (%p - %" GST_TIME_FORMAT
+      ") %s%15s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
+      gst_debug_level_get_name (level), g_thread_self (),
+      GST_TIME_ARGS (GST_TIMEVAL_TO_TIME (now)), color,
+      gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
       color, file, line, function, obj, clear, gst_debug_message_get (message));
 
   g_free (color);
@@ -797,8 +800,8 @@ for_each_threshold_by_entry (gpointer data, gpointer user_data)
  * gst_debug_set_threshold_for_name:
  * @name: name of the categories to set
  * @level: level to set them to
- * 
- * Sets all categories which match the gven glob style pattern to the given 
+ *
+ * Sets all categories which match the given glob style pattern to the given
  * level.
  */
 void
@@ -824,7 +827,7 @@ gst_debug_set_threshold_for_name (const gchar * name, GstDebugLevel level)
 /**
  * gst_debug_unset_threshold_for_name:
  * @name: name of the categories to set
- * 
+ *
  * Resets all categories with the given name back to the default level.
  */
 void
@@ -1104,8 +1107,17 @@ _gst_info_printf_extension_arginfo (const struct printf_info *info, size_t n,
 }
 #endif /* HAVE_PRINTF_EXTENSION */
 
+#else /* !GST_DISABLE_GST_DEBUG */
+
+gboolean
+__gst_in_valgrind (void)
+{
+  return FALSE;
+}
+
 #endif /* GST_DISABLE_GST_DEBUG */
 
+
 #ifdef GST_ENABLE_FUNC_INSTRUMENTATION
 /* FIXME make this thread specific */
 static GSList *stack_trace = NULL;
