@@ -36,9 +36,9 @@
 
 G_BEGIN_DECLS
 
-extern GType _gst_pad_type;
-extern GType _gst_real_pad_type;
-extern GType _gst_ghost_pad_type;
+GST_EXPORT GType _gst_pad_type;
+GST_EXPORT GType _gst_real_pad_type;
+GST_EXPORT GType _gst_ghost_pad_type;
 
 /* 
  * Pad base class
@@ -140,6 +140,7 @@ typedef enum {
 typedef enum {
   GST_PAD_DISABLED		= GST_OBJECT_FLAG_LAST,
   GST_PAD_NEGOTIATING,
+  GST_PAD_DISPATCHING,
 
   GST_PAD_FLAG_LAST		= GST_OBJECT_FLAG_LAST + 4
 } GstPadFlags;
@@ -280,6 +281,7 @@ struct _GstGhostPadClass {
 #define GST_PAD_IS_LINKED(pad)		(GST_PAD_PEER(pad) != NULL)
 #define GST_PAD_IS_ACTIVE(pad)		(!GST_FLAG_IS_SET(GST_PAD_REALIZE(pad), GST_PAD_DISABLED))
 #define GST_PAD_IS_NEGOTIATING(pad)	(GST_FLAG_IS_SET (pad, GST_PAD_NEGOTIATING))
+#define GST_PAD_IS_DISPATCHING(pad)	(GST_FLAG_IS_SET (pad, GST_PAD_DISPATCHING))
 #define GST_PAD_IS_USABLE(pad)		(GST_PAD_IS_LINKED (pad) && \
 		                         GST_PAD_IS_ACTIVE(pad) && GST_PAD_IS_ACTIVE(GST_PAD_PEER (pad)))
 #define GST_PAD_CAN_PULL(pad)		(GST_IS_REAL_PAD(pad) && GST_REAL_PAD(pad)->gethandler != NULL)
