@@ -215,6 +215,7 @@ void
 gst_scheduler_add_element (GstScheduler * sched, GstElement * element)
 {
   GstSchedulerClass *sclass;
+  gboolean redistribute_clock = FALSE;
 
   g_return_if_fail (GST_IS_SCHEDULER (sched));
   g_return_if_fail (GST_IS_ELEMENT (element));
@@ -233,15 +234,24 @@ gst_scheduler_add_element (GstScheduler * sched, GstElement * element)
     sched->clock_providers = g_list_prepend (sched->clock_providers, element);
     GST_CAT_DEBUG (GST_CAT_CLOCK, "added clock provider %s",
         GST_ELEMENT_NAME (element));
+    redistribute_clock = TRUE;
   }
   if (gst_element_requires_clock (element)) {
     sched->clock_receivers = g_list_prepend (sched->clock_receivers, element);
     GST_CAT_DEBUG (GST_CAT_CLOCK, "added clock receiver %s",
         GST_ELEMENT_NAME (element));
+    redistribute_clock = TRUE;
   }
 
   gst_element_set_scheduler (element, sched);
 
+  if (redistribute_clock) {
+    GstClock *clock;
+
+    clock = gst_scheduler_get_clock (sched);
+    gst_scheduler_set_clock (sched, clock);
+  }
+
   sclass = GST_SCHEDULER_GET_CLASS (sched);
 
   if (sclass->add_element)
@@ -259,12 +269,33 @@ void
 gst_scheduler_remove_element (GstScheduler * sched, GstElement * element)
 {
   GstSchedulerClass *sclass;
+  GList *link;
+  gboolean redistribute_clock = FALSE;
 
   g_return_if_fail (GST_IS_SCHEDULER (sched));
   g_return_if_fail (GST_IS_ELEMENT (element));
 
-  sched->clock_providers = g_list_remove (sched->clock_providers, element);
-  sched->clock_receivers = g_list_remove (sched->clock_receivers, element);
+  link = g_list_find (sched->clock_providers, element);
+  if (link) {
+    sched->clock_providers = g_list_delete_link (sched->clock_providers, link);
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "removed clock provider %s",
+        GST_ELEMENT_NAME (element));
+    redistribute_clock = TRUE;
+  }
+  link = g_list_find (sched->clock_receivers, element);
+  if (link) {
+    sched->clock_receivers = g_list_delete_link (sched->clock_receivers, link);
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "removed clock receiver %s",
+        GST_ELEMENT_NAME (element));
+    redistribute_clock = TRUE;
+  }
+
+  if (redistribute_clock) {
+    GstClock *clock;
+
+    clock = gst_scheduler_get_clock (sched);
+    gst_scheduler_set_clock (sched, clock);
+  }
 
   sclass = GST_SCHEDULER_GET_CLASS (sched);
 
@@ -295,6 +326,8 @@ gst_scheduler_state_transition (GstScheduler * sched, GstElement * element,
   g_return_val_if_fail (GST_IS_ELEMENT (element), GST_STATE_FAILURE);
 
   if (element == sched->parent && sched->parent_sched == NULL) {
+    /* FIXME is distributing the clock in the state change still needed
+     * when we distribute as soon as we add/remove elements? I think not.*/
     switch (transition) {
       case GST_STATE_READY_TO_PAUSED:
       {
@@ -793,6 +826,45 @@ gst_scheduler_factory_init (GstSchedulerFactory * factory)
 }
 
 
+/**
+ * gst_scheduler_register:
+ * @plugin: a #GstPlugin
+ * @name: name of the scheduler to register
+ * @longdesc: description of the scheduler
+ * @type: #GType of the scheduler to register
+ *
+ * Registers a scheduler with GStreamer.
+ *
+ * Returns: TRUE, if the registering succeeded, FALSE on error.
+ *
+ * Since: 0.8.5
+ **/
+gboolean
+gst_scheduler_register (GstPlugin * plugin, const gchar * name,
+    const gchar * longdesc, GType type)
+{
+  GstSchedulerFactory *factory;
+
+  g_return_val_if_fail (plugin != NULL, FALSE);
+  g_return_val_if_fail (name != NULL, FALSE);
+  g_return_val_if_fail (longdesc != NULL, FALSE);
+  g_return_val_if_fail (g_type_is_a (type, GST_TYPE_SCHEDULER), FALSE);
+
+  factory = gst_scheduler_factory_find (name);
+  if (factory) {
+    g_return_val_if_fail (factory->type == 0, FALSE);
+    g_free (factory->longdesc);
+    factory->longdesc = g_strdup (longdesc);
+    factory->type = type;
+  } else {
+    factory = gst_scheduler_factory_new (name, longdesc, type);
+    g_return_val_if_fail (factory, FALSE);
+    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  }
+
+  return TRUE;
+}
+
 /**
  * gst_scheduler_factory_new:
  * @name: name of schedulerfactory to create
