@@ -482,7 +482,7 @@ plugin_times_older_than_recurse (gchar * path, time_t regtime)
 
   /* it's a file, and it's more recent */
   if (g_str_has_suffix (path, ".so") || g_str_has_suffix (path, ".dll")) {
-    if (! gst_plugin_check_file (path, NULL))
+    if (!gst_plugin_check_file (path, NULL))
       return TRUE;
 
     /* it's a newer GStreamer plugin */
@@ -581,7 +581,7 @@ gst_xml_registry_open_func (GstXMLRegistry * registry, GstXMLRegistryMode mode)
 
     GST_CAT_DEBUG (GST_CAT_GST_INIT, "opening registry %s for writing", tmploc);
 
-    if ((fd = open (tmploc, O_WRONLY | O_CREAT | O_EXCL, 0644)) < 0) {
+    if ((fd = open (tmploc, O_WRONLY | O_CREAT, 0644)) < 0) {
       g_free (tmploc);
       return FALSE;
     }
@@ -631,8 +631,12 @@ gst_xml_registry_close_func (GstXMLRegistry * registry)
 
   /* If we opened for writing, rename our temporary file. */
   tmploc = g_strconcat (registry->location, ".tmp", NULL);
-  if (g_file_test (tmploc, G_FILE_TEST_EXISTS))
+  if (g_file_test (tmploc, G_FILE_TEST_EXISTS)) {
+#ifdef WIN32
+    remove (registry->location);
+#endif
     rename (tmploc, registry->location);
+  }
   g_free (tmploc);
 
   registry->open = FALSE;
