@@ -1,7 +1,7 @@
 
 #include <gst/gst.h>
 
-int 
+int
 main (int argc, char *argv[])
 {
   GstPlugin *plugin;
@@ -10,15 +10,14 @@ main (int argc, char *argv[])
   gst_init (&argc, &argv);
 
   plugin = gst_plugin_load_file (".libs/libtestplugin.so", &error);
-  g_assert (plugin != NULL);
-  if (error)
-  {
+  if (error) {
     g_print ("ERROR loading plug-in: %s\n", error->message);
     g_free (error);
     return 1;
   }
+  g_assert (plugin != NULL);
 
-  g_print ("testplugin: %s\n", gst_plugin_get_name(plugin));
+  g_print ("testplugin: %s\n", gst_plugin_get_name (plugin));
 
   return 0;
 }
